/**************************************************************
 * This is a program to control ON or OFF 8 outputs in a      *
 * Arduino PCB throught the etherShield sold on the online    *
 * store www.nuelectronics.com using their own library for    *
 * controlling 8 relays.                                      *
 *                                                            *
 * Also the whole program is based on the ethernet_web_switch *
 * example avaiable in the etherShield library.               *
 *                                                            *
 *           GNU GPLv2 Licensed, by KaR]V[aN                  *
 *                                                            *
 * Usage: put http://192.168.0.123/password/ on your browser. *
 **************************************************************/
 
 /* This code is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
 
#include "etherShield.h"
#include "string.h"
#include "stdio.h"

// Mac address, must be unique on the local area network
static uint8_t mymac[6] = {0x54,0x55,0x58,0x10,0x00,0x24}; 

// IP address, must be unique on the local area network
static uint8_t myip[4] = {192,168,0,13}; 

// Some password
char password[] = "password";  

// Names for the pins outputs, the first the pin 2, the last the pin 9
char *nombres[] = {"Modem","SAI","Server","Access Point 1","Switch","Access Point 2","Access Point 3","AUX"};  

// listen port for tcp/www (max range 1-254)
static uint8_t mywwwport = 81; // listen port for tcp/www (max range 1-254)

/* NOTE: You should be careful about the length of *nombres and password. If they are too much 
         large the page could not fit in the 500 bytes buffer length of the tcp packet.
         You will know if its too much large if your browser hangs trying to load any page. */



/*********************
 * DO NOT EDIT BELOW * 
 *********************/
char baseurl[sizeof(password)+2];
boolean status[8] = {false,false,false,false,false,false,false,false};
char *codes[8][2] = {{"82","83"},{"12","13"},{"22","23"},{"32","33"},{"42","43"},{"52","53"},{"62","63"},{"72","73"}};
//char *led_[] = {"1","2","3","4","5","6","7","8"};

#define BUFFER_SIZE 500
static uint8_t buf[BUFFER_SIZE+1];
#define STR_BUFFER_SIZE 22
static char strbuf[STR_BUFFER_SIZE+1];

EtherShield es=EtherShield();

// LED cathode connects the +5V, anode to GND through 1K resistor
// 2 to 9
#define LED_PIN_1 2
#define LED_PIN_2 3
#define LED_PIN_3 4
#define LED_PIN_4 5
#define LED_PIN_5 6
#define LED_PIN_6 7
#define LED_PIN_7 8
#define LED_PIN_8 9



void setup(){
  
   /*initialize enc28j60*/
	 es.ES_enc28j60Init(mymac);
   es.ES_enc28j60clkout(2); // change clkout from 6.25MHz to 12.5MHz
   delay(10);
        
	/* Magjack leds configuration, see enc28j60 datasheet, page 11 */
	// LEDA=greed LEDB=yellow

	//
	// 0x880 is PHLCON LEDB=on, LEDA=on
	// enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00);
//	delay(500);
	//
	// 0x990 is PHLCON LEDB=off, LEDA=off
	// enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00);
//	es.ES_enc28j60PhyWrite(PHLCON,0x990);
//	delay(500);
	//
	// 0x880 is PHLCON LEDB=on, LEDA=on
	// enc28j60PhyWrite(PHLCON,0b0000 1000 1000 00 00);
//	es.ES_enc28j60PhyWrite(PHLCON,0x880);
//	delay(500);
	//
	// 0x990 is PHLCON LEDB=off, LEDA=off
	// enc28j60PhyWrite(PHLCON,0b0000 1001 1001 00 00);
//	es.ES_enc28j60PhyWrite(PHLCON,0x990);
//	delay(500);
	//
        
  // 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
  // enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10);
        es.ES_enc28j60PhyWrite(PHLCON,0x476);
        delay(100);
        
  //init the ethernet/ip layer:
  es.ES_init_ip_arp_udp_tcp(mymac,myip,mywwwport);
  
 	pinMode(LED_PIN_1, OUTPUT); 
 	digitalWrite(LED_PIN_1, LOW); 
        pinMode(LED_PIN_2, OUTPUT); 
 	digitalWrite(LED_PIN_2, LOW);
        pinMode(LED_PIN_3, OUTPUT); 
 	digitalWrite(LED_PIN_3, LOW);
        pinMode(LED_PIN_4, OUTPUT); 
 	digitalWrite(LED_PIN_4, LOW);
        pinMode(LED_PIN_5, OUTPUT); 
 	digitalWrite(LED_PIN_5, LOW);
 	pinMode(LED_PIN_6, OUTPUT); 
 	digitalWrite(LED_PIN_6, LOW);
 	pinMode(LED_PIN_7, OUTPUT); 
 	digitalWrite(LED_PIN_7, LOW);
 	pinMode(LED_PIN_8, OUTPUT); 
 	digitalWrite(LED_PIN_8, LOW);
}

void loop(){
  uint16_t plen, dat_p;
  int8_t cmd, led;

  // Obtain the actual base url
  sprintf(baseurl,"/%s/",password);

  plen = es.ES_enc28j60PacketReceive(BUFFER_SIZE, buf);

	/*plen will ne unequal to zero if there is a valid packet (without crc error) */
  if(plen!=0){
	           
    // arp is broadcast if unknown but a host may also verify the mac address by sending it to a unicast address.
    if(es.ES_eth_type_is_arp_and_my_ip(buf,plen)){
      es.ES_make_arp_answer_from_request(buf);
      return;
    }

    // check if ip packets are for us:
    if(es.ES_eth_type_is_ip_and_my_ip(buf,plen)==0){
      return;
    }
    
    if(buf[IP_PROTO_P]==IP_PROTO_ICMP_V && buf[ICMP_TYPE_P]==ICMP_TYPE_ECHOREQUEST_V){
      es.ES_make_echo_reply_from_request(buf,plen);
      return;
    }
    
    // tcp port www start, compare only the lower byte
    if (buf[IP_PROTO_P]==IP_PROTO_TCP_V&&buf[TCP_DST_PORT_H_P]==0&&buf[TCP_DST_PORT_L_P]==mywwwport){
      if (buf[TCP_FLAGS_P] & TCP_FLAGS_SYN_V){
         es.ES_make_tcp_synack_from_syn(buf); // make_tcp_synack_from_syn does already send the syn,ack
         return;     
      }
      if (buf[TCP_FLAGS_P] & TCP_FLAGS_ACK_V){
        es.ES_init_len_info(buf); // init some data structures
        dat_p=es.ES_get_tcp_data_pointer();
        if (dat_p==0){ // we can possibly have no data, just ack:
          if (buf[TCP_FLAGS_P] & TCP_FLAGS_FIN_V){
            es.ES_make_tcp_ack_from_any(buf);
          }
          return;
        }
        if (strncmp("GET ",(char *)&(buf[dat_p]),4)!=0){  // If its not a GET..
          	// head, post and other methods for possible status codes see:
            // http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html
            plen=es.ES_fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n<h1>200 OK</h1>"));
            goto SENDTCP;
        }
        
        //  What are we going to search?
        char pass[sizeof(baseurl)+1];
        char pass2[sizeof(baseurl)+2];
        char passqm[sizeof(baseurl)+1];
        
        sprintf(pass,"%s ",baseurl);    // First page of relays, "GET /password/ "
        sprintf(pass2,"%s2 ",baseurl);  // Second page of relays, "GET /password/2 "
        sprintf(passqm,"%s?",baseurl);  // Page of indivindual relay or changes, "GET /password/?[cmd=|led=]"
        
 	if (strncmp(pass,(char *)&(buf[dat_p+4]),strlen(pass))==0) {
                plen=print_webpage(buf, status, 1);
        } else if (strncmp(pass2,(char *)&(buf[dat_p+4]),strlen(pass2))==0) {
                plen=print_webpage(buf, status, 5); 
        } else if (strncmp(passqm,(char *)&(buf[dat_p+4]),strlen(passqm))==0) {
                
                //   GET /pass/?cmd=lol HTTP
                cmd=analyse_cmd((char *)&(buf[dat_p+strlen(baseurl)+4]));
                led=analyse_led((char *)&(buf[dat_p+strlen(baseurl)+4]));
  
               if (led > 0) {
                    //plen=print_webpage_led(buf, status[led-1], led_[led-1], codes[led-1]);
                     plen=print_webpage_led(buf, status[led-1], led, codes[led-1]);
                } else if(cmd > 0) {
                    if (cmd==82){
                        status[0]=true;
                        digitalWrite(LED_PIN_1, HIGH);  // switch on LED
                        plen=print_webpage_led(buf, status[0], 1, codes[0]);
                    }
                    else if (cmd==83){
                        status[0]=false;
              	        digitalWrite(LED_PIN_1, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[0], 1, codes[0]);
                    }
                    else if (cmd==12){
                        status[1]=true;
              	        digitalWrite(LED_PIN_2, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[1], 2, codes[1]);
                    }
                    else if (cmd==13){
                        status[1]=false;
              	        digitalWrite(LED_PIN_2, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[1], 2, codes[1]);
                    }
                    else if (cmd==22){
                        status[2]=true;
              	        digitalWrite(LED_PIN_3, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[2], 3, codes[2]);
                    }
                    else if (cmd==23){
                        status[2]=false;
              	        digitalWrite(LED_PIN_3, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[2], 3, codes[2]);
                    }
                    else if (cmd==32){
                        status[3]=true;
              	        digitalWrite(LED_PIN_4, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[3], 4, codes[3]);
                    }
                    else if (cmd==33){
                        status[3]=false;
              	        digitalWrite(LED_PIN_4, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[3], 4, codes[3]);
                    }
                    else if (cmd==42){
                        status[4]=true;
              	        digitalWrite(LED_PIN_5, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[4], 5, codes[4]);
                    }
                    else if (cmd==43){
                        status[4]=false;
              	        digitalWrite(LED_PIN_5, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[4], 5, codes[4]);
                    }
                    else if (cmd==52){
                        status[5]=true;
              	        digitalWrite(LED_PIN_6, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[5], 6, codes[5]);
                    }
                    else if (cmd==53){
                        status[5]=false;
              	        digitalWrite(LED_PIN_6, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[5], 6, codes[5]);
                    }
                    else if (cmd==62){
                        status[6]=true;
              	        digitalWrite(LED_PIN_7, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[6], 7, codes[6]);
                    }
                    else if (cmd==63){
                        status[6]=false;
              	        digitalWrite(LED_PIN_7, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[6], 7, codes[6]);
                    }
                    else if (cmd==72){
                        status[7]=true;
              	        digitalWrite(LED_PIN_8, HIGH);  // switch off LED
                        plen=print_webpage_led(buf, status[7], 8, codes[7]);
                    }
                    else if (cmd==73){
                        status[7]=false;
              	        digitalWrite(LED_PIN_8, LOW);  // switch off LED
                        plen=print_webpage_led(buf, status[7], 8, codes[7]);
                    }
                } else 
                    plen=print_webpage_huh(buf);
        } else 
                plen=print_webpage_un(buf);
        	

SENDTCP:  es.ES_make_tcp_ack_from_any(buf); // send ack for http get
          es.ES_make_tcp_ack_with_data(buf,plen); // send data       
      }
    }
  }
        
}
// The returned value is stored in the global var strbuf
uint8_t find_key_val(char *str,char *key)
{
        uint8_t found=0;
        uint8_t i=0;
        char *kp;
        kp=key;
        while(*str &&  *str!=' ' && found==0){
                if (*str == *kp){
                        kp++;
                        if (*kp == '\0'){
                                str++;
                                kp=key;
                                if (*str == '='){
                                        found=1;
                                }
                        }
                }else{
                        kp=key;
                }
                str++;
        }
        if (found==1){
                // copy the value to a buffer and terminate it with '\0'
                while(*str &&  *str!=' ' && *str!='&' && i<STR_BUFFER_SIZE){
                        strbuf[i]=*str;
                        i++;
                        str++;
                }
                strbuf[i]='\0';
        }
        return(found);
}

int8_t analyse_cmd(char *str)
{
        int8_t r=-1;
        char tmp[4];
        if (find_key_val(str,"cmd")){
                if (*strbuf < 0x3a && *strbuf > 0x2f){
                        // is a ASCII number, return it
                       //r=(*strbuf-0x30);
                       sprintf(tmp,"%i%i",(strbuf[0]-0x30),(strbuf[1]-0x30));
                       r=atoi(tmp);
                       
                }
        }
        return r;
}

int8_t analyse_led(char *str)
{
        int8_t r=-1;
        
        if (find_key_val(str,"led")){
                if (*strbuf < 0x3a && *strbuf > 0x2f){
                        // is a ASCII number, return it
                        r=(*strbuf-0x30);
                }
        }
        return r;
}

//uint16_t print_webpage_led(uint8_t *buf, boolean status, char *led, char *codes[])
uint16_t print_webpage_led(uint8_t *buf, boolean status, int led, char *codes[])
{
   
        uint16_t plen;
       
        plen=es.ES_fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
         
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<html><body><center><form METHOD=get action=\""));
        plen=es.ES_fill_tcp_data(buf,plen,baseurl);
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("\">\n"));
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<table><tr><td><h3>#PIN</h3></td> <td><h3>Status</h3></td> <td><h3>Change</h3></td></tr>"));
        
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<tr><td>"));
        plen=es.ES_fill_tcp_data(buf,plen,nombres[led-1]);
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR(":</td><td><b><font color=\""));
        if(status==true)
        	plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("#00FF00\">ON"));
        else 
        	plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("#FF0000\">OFF"));
        
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("</font></b></td><td><input type=hidden name=cmd value="));
        if(status==true) {
        	plen=es.ES_fill_tcp_data(buf,plen,codes[1]);
        	plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("><input type=submit value=\"Switch off\">"));
        } else {
        	plen=es.ES_fill_tcp_data(buf,plen,codes[0]);
        	plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("><input type=submit value=\"Switch on\">"));
        }
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("</td>"));
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("</tr></table></form>"));
        if(led < 5) {
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<a href=\""));
          plen=es.ES_fill_tcp_data(buf,plen,baseurl);
        } else {
          char tmp[sizeof(baseurl)+1];
          sprintf(tmp,"%s%s",baseurl,"2");
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<a href=\""));
          plen=es.ES_fill_tcp_data(buf,plen,tmp);
        }  
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("\">Go back</a>"));
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("</center></body></html>"));       
  
        return(plen);
}

uint16_t print_webpage(uint8_t *buf, boolean status[], int start)
{
   
        uint16_t plen;
       
        plen=es.ES_fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
         
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<html><body><center>"));
        
        for(int i=start;i<start+4;i++) {
          char num[1];
          sprintf(num,"%i",i);
          
          plen=es.ES_fill_tcp_data(buf,plen,nombres[i-1]);
          //plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("LED"));
          //plen=es.ES_fill_tcp_data(buf,plen,num);
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR(": <a href=\""));
          plen=es.ES_fill_tcp_data(buf,plen,baseurl);
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("?led="));
          plen=es.ES_fill_tcp_data(buf,plen,num);
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("\">"));
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<b><font color=\""));
          if(status[i-1]==true)
          	plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("#00FF00\">ON"));
          else 
          	plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("#FF0000\">OFF"));
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("</font></b></a>"));
        
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<br>"));
        }
        
        if(start == 5) {
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<a href=\""));
          plen=es.ES_fill_tcp_data(buf,plen,baseurl);
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("\">Go index</a>"));
        } else {
          char tmp[sizeof(baseurl)+1];
          sprintf(tmp,"%s%s",baseurl,"2");
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<a href=\""));
          plen=es.ES_fill_tcp_data(buf,plen,tmp);
          plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("\">Go second page</a>"));
        }  
      
        return(plen);
}

uint16_t print_webpage_un(uint8_t *buf)
{
        uint16_t plen;
       
        plen=es.ES_fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<html><head><title>g0 h0m3</title></head><body><center><h1>You are not authorized</h1></center></body></html>"));
        
        return(plen);
}

uint16_t print_webpage_huh(uint8_t *buf)
{
        uint16_t plen;
       
        plen=es.ES_fill_tcp_data_p(buf,0,PSTR("HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n"));
        plen=es.ES_fill_tcp_data_p(buf,plen,PSTR("<html><head><title>Huh?</title></head><body><center><h1>I dont understand you</h1></center></body></html>"));
        
        return(plen);
}
